@@ -14,13 +14,15 @@ import py7zr
 from . import __app_name__, __version__
 from .client import ToDusClient
 
-logging.basicConfig(
-    format="%(asctime)s-%(levelname)s-%(name)s-%(message)s", level=logging.INFO
-)
+formatter = logging.Formatter("%(levelname)s-%(name)s-%(asctime)s-%(message)s")
 logger = logging.getLogger(__name__)
 logger.setLevel(logging.INFO)
+handler_default = logging.StreamHandler()
+handler_default.setFormatter(formatter)
 handler_error = logging.FileHandler("logs", encoding="utf-8")
 handler_error.setLevel(logging.ERROR)
+handler_error.setFormatter(formatter)
+logger.addHandler(handler_default)
 logger.addHandler(handler_error)
 
 client = ToDusClient()
@@ -44,7 +46,11 @@ def split_upload(
 
     with open(path, "rb") as file:
         data = file.read()
+
     filename = Path(path).name
+
+    logger.info("Compressing parts...")
+
     with TemporaryDirectory() as tempdir:
         with multivolumefile.open(
             Path(f"{tempdir}/{filename}.7z"),
@@ -63,18 +69,18 @@ def split_upload(
             retry = 0
             up_done = False
             logger.info(f"Uploading {i}/{parts_count}: {name}")
-            with open(Path(f"{tempdir}/{name}"), "rb") as file:
-                part = file.read()
+
+            temp_path = Path(f"{tempdir}/{name}")
 
             while not up_done and retry < max_retry:
                 try:
-                    urls.append(client.upload_file(token, part, len(part)))
+                    urls.append(client.upload_file(token, temp_path, index=i))
                 except Exception as ex:
-                    logger.exception(ex)
+                    logger.error(ex)
                     retry += 1
                     if retry == max_retry:
                         raise ValueError(
-                            f"Failed to upload part {i} ({len(part):,}B): {ex}"
+                            f"Failed to upload part {i} ({temp_path.stat().st_size:,}B): {ex}"
                         )
                     logger.info(f"Retrying: {retry}...")
                     time.sleep(15)
@@ -171,9 +177,8 @@ def _upload(token: str, args: argparse.Namespace, max_retry: int):
             logger.info(f"TXT: {txt}")
         else:
             # TODO: Retry for single file
-            with open(path, "rb") as file:
-                data = file.read()
-            file_uri = client.upload_file(token, data, len(data))
+            filename_path = Path(path)
+            file_uri = client.upload_file(token, filename_path)
             down_url = f"{file_uri}?name={quote_plus(filename)}"
             logger.info(f"URL: {down_url}")
             txt = write_txt(filename, urls=[file_uri], parts=[filename])
@@ -183,14 +188,16 @@ def _upload(token: str, args: argparse.Namespace, max_retry: int):
 def _download(
     token: str, args: argparse.Namespace, down_timeout: float, max_retry: int
 ):
+    urls = []
+
     while args.url:
         retry = 0
         down_done = False
         file_uri = args.url.pop(0)
 
+        # Extract URLS from current TXT
         if os.path.exists(file_uri):
             with open(file_uri) as fp:
-                urls = []
                 for line in fp.readlines():
                     line = line.strip()
                     if line:
@@ -198,8 +205,14 @@ def _download(
                         urls.append(f"{_url}?name={_filename}")
 
                 args.url = urls + args.url
+                txt_process = True
                 continue
 
+        if urls:
+            count_files = len(urls)
+            plural = "" if count_files <= 1 else "s"
+            logger.info(f"Downloading: {count_files} file{plural}")
+
         logger.info(
             f"Downloading: {file_uri}",
         )
@@ -211,7 +224,7 @@ def _download(
             try:
                 size = client.download_file(token, file_uri, name, down_timeout)
             except Exception as ex:
-                logger.exception(ex)
+                logger.error(ex)
 
             if size:
                 logger.debug(
@@ -226,6 +239,8 @@ def _download(
                 logger.info(f"Retrying: {retry}...")
                 time.sleep(15)
 
+        urls = []
+
 
 def main() -> None:
     global client, config, logger
